@@ -191,7 +191,10 @@ bool handle_chld_out_event(int fd)
      }
      else
      {
-          D("[CHLD]:%s\n", linebuf);
+          if(pos > 0)
+          {
+              D("[CHLD]:%s\n", linebuf);
+          }
           pos = 0;
           return false;
      }
