@@ -57,11 +57,16 @@ struct AppData_s
 
 typedef struct AppData_s AppData_t;
 
-/******************************************************************************/
+struct SigGlobals_s
+{
+    Display * dpy;
+    pid_t childPid;
+};
 
-static pid_t pid = -1; /* Has to be global */
+typedef struct SigGlobals_s SigGlobals_t;
 
 
+static SigGlobals_t sig_globals;
 static GC gc_white;
 static GC gc_black;
 static GC gc_onColor;
@@ -82,7 +87,7 @@ static void redrawButton(Display * dpy, Window win, int buttonDown)
 
      XWindowAttributes attr;
      XPoint base;
-     GC c = (pid > 0) ? gc_onColor : gc_offColor;
+     GC c = (sig_globals.childPid > 0) ? gc_offColor : gc_onColor;
 
      XGetWindowAttributes(dpy, win, &attr);
      bsize = attr.width;
@@ -128,13 +133,13 @@ static void terminate(pid_t pid, Display * dpy)
 /**
  * Called when the user clicks on the play button
  *
- * @param[in] dpy The display pointer
  * @param[in] win The window ID
  * @param[in] data The Application details
  */
-static void my_start(Display * dpy, Window win, AppData_t * data)
+static void my_start(Window win, AppData_t * data)
 {
-     if(pid > 0)
+     Display * dpy = sig_globals.dpy;
+     if(sig_globals.childPid > 0)
      {
           return;
      }
@@ -171,7 +176,7 @@ static void my_start(Display * dpy, Window win, AppData_t * data)
      }
 
      /* else linker is used to launch command */
-     pid = spawn_app(data->command, data->flags);
+     sig_globals.childPid = spawn_app(data->command, data->flags);
 }
 
 /**
@@ -179,10 +184,7 @@ static void my_start(Display * dpy, Window win, AppData_t * data)
  */
 static void low_die(void)
 {
-     if(pid > 0)
-     {
-           kill_app(pid);
-     }
+     terminate(sig_globals.childPid, sig_globals.dpy);
      _exit(0);
 }
 
@@ -230,15 +232,15 @@ static void sigdie(int sig)
  * Checks for X events and processes them. Returns when no more events left
  * to process.
  *
- * @param[in] dpy The Display pointer
  * @param[in] win The window ID
  * @param[in] data The Application data
  * @param[in] doDraw
  *
  * @return nothing
  */
-static void check_x_events(Display * dpy, Window win, AppData_t * data, int doDraw)
+static void check_x_events(Window win, AppData_t * data, int doDraw)
 {
+     Display * dpy = sig_globals.dpy;
      int numEvents = XPending(dpy);
      static int buttonDown = 0;
 
@@ -257,14 +259,14 @@ static void check_x_events(Display * dpy, Window win, AppData_t * data, int doDr
           case ButtonPress:
                if(ev.xbutton.button == 1)
                {
-                    pid_t tmp = pid;
+                    pid_t tmp = sig_globals.childPid;
                     buttonDown = 1;
                     redrawButton(dpy, win, buttonDown); /* do first gives quicker visual feedback */
                     XSync(dpy, False);
 
-                    my_start(dpy, win, data);
+                    my_start(win, data);
 
-                    if(tmp != pid)
+                    if(tmp != sig_globals.childPid)
                     {
                         redrawButton(dpy, win, buttonDown); /* do first gives quicker visual feedback */
                     }
@@ -347,11 +349,10 @@ static void normalise_window_coords(unsigned * width, unsigned * height,
  * Check to see if new window size information has arrived from plugin and if
  * so resize the button accordingly
  *
- * @param[in] dpy The Display pointer
  * @param[in] win The window ID
  * @param[in] data The Application data
  */
-static void check_pipe_fd_events(Display * dpy, Window win, AppData_t * data)
+static void check_pipe_fd_events(Window win, AppData_t * data)
 {
      struct PipeMsg_s msg;
      int n;
@@ -364,7 +365,7 @@ static void check_pipe_fd_events(Display * dpy, Window win, AppData_t * data)
      if((n == 0) || ((n < 0) && (errno != EINTR)))
      {
           D("Pipe returned n=%i\n", n);
-          terminate(pid, dpy);
+          terminate(sig_globals.childPid, sig_globals.dpy);
 	  exit(EX_UNAVAILABLE);
      }
 
@@ -393,7 +394,7 @@ static void check_pipe_fd_events(Display * dpy, Window win, AppData_t * data)
 
                D("Linker window: x=%i, y=%i, w=%u, h=%u\n", x, y, w, h);
 
-               XMoveResizeWindow(dpy, win, x, y, w, h);
+               XMoveResizeWindow(sig_globals.dpy, win, x, y, w, h);
           }
           break;
      }
@@ -589,6 +590,8 @@ int main(int argc, char **argv)
      sig_chld_fd = redirect_SIGCHLD_to_fd();
      rd_chld_fd = get_chld_out_fd();
 
+     sig_globals.childPid = -1;
+     sig_globals.dpy = dpy;
      signal(SIGHUP, sigdie);
      signal(SIGINT, sigdie);
      signal(SIGTERM, sigdie);
@@ -598,11 +601,11 @@ int main(int argc, char **argv)
           fd_set fds;
           int maxFd;
 
-          check_x_events(dpy, topLevel, &data, 0);
+          check_x_events(topLevel, &data, 0);
 
           FD_ZERO(&fds);
-          FD_SET(ConnectionNumber(dpy),&fds);
-          maxFd = ConnectionNumber(dpy);
+          FD_SET(ConnectionNumber(sig_globals.dpy),&fds);
+          maxFd = ConnectionNumber(sig_globals.dpy);
 
           FD_SET(data.pipe_fd, &fds);
           maxFd = data.pipe_fd > maxFd ? data.pipe_fd : maxFd;
@@ -624,7 +627,7 @@ int main(int argc, char **argv)
           {
                if (FD_ISSET(data.pipe_fd, &fds))
                {
-                    check_pipe_fd_events(dpy, topLevel, &data);
+                    check_pipe_fd_events(topLevel, &data);
                }
                if(FD_ISSET(sig_chld_fd, &fds))
                {
@@ -638,22 +641,22 @@ int main(int argc, char **argv)
 
           D("SELECT OUT\n");
 
-          if(pid != -1)
+          if(sig_globals.childPid > 0)
           {
-               int status = wait_child(pid);
+               int status = wait_child(sig_globals.childPid);
                if(status <= 0 )
                {
-                    pid = -1;
+                    sig_globals.childPid = -1;
                     if(data.repeats != INF_LOOPS)
                     {
                          repeatsLeft--;
                     }
                     if(repeatsLeft > 0)
                     {
-                         my_start(dpy, topLevel, &data);
+                         my_start(topLevel, &data);
                          rd_chld_fd = get_chld_out_fd();
                     }
-                    check_x_events(dpy, topLevel, &data, 1);
+                    check_x_events(topLevel, &data, 1);
                }
           }
           else
